"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FacebookLoginController = void 0;
const tslib_1 = require("tslib");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_1 = require("@sourceloop/core");
const jwt = (0, tslib_1.__importStar)(require("jsonwebtoken"));
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const url_1 = require("url");
const providers_1 = require("../../providers");
const repositories_1 = require("../../repositories");
const client_auth_request_dto_1 = require("./models/client-auth-request.dto");
const token_response_dto_1 = require("./models/token-response.dto");
const queryGen = (from) => {
    return (req) => {
        return {
            state: `client_id=${req[from].client_id}`,
        };
    };
};
let FacebookLoginController = class FacebookLoginController {
    constructor(authClientRepository, logger) {
        this.authClientRepository = authClientRepository;
        this.logger = logger;
    }
    async postLoginViaFacebook(clientCreds) { }
    async facebookCallback(code, state, response, facebookCodeWriter, user) {
        const clientId = new url_1.URLSearchParams(state).get('client_id');
        if (!clientId || !user) {
            throw new rest_1.HttpErrors.Unauthorized("Client Invalid" /* ClientInvalid */);
        }
        const client = await this.authClientRepository.findOne({
            where: {
                clientId,
            },
        });
        if (!client || !client.redirectUrl) {
            throw new rest_1.HttpErrors.Unauthorized("Client Invalid" /* ClientInvalid */);
        }
        try {
            const codePayload = {
                clientId,
                user: user,
            };
            const token = await facebookCodeWriter(jwt.sign(codePayload, client.secret, {
                expiresIn: client.authCodeExpiration,
                audience: clientId,
                issuer: process.env.JWT_ISSUER,
                algorithm: 'HS256',
            }));
            response.redirect(`${client.redirectUrl}?code=${token}`);
        }
        catch (error) {
            this.logger.error(error);
            throw new rest_1.HttpErrors.Unauthorized("Invalid Credentials" /* InvalidCredentials */);
        }
    }
};
(0, tslib_1.__decorate)([
    (0, loopback4_authentication_1.authenticateClient)("client-password" /* CLIENT_PASSWORD */),
    (0, loopback4_authentication_1.authenticate)("Facebook Oauth 2.0" /* FACEBOOK_OAUTH2 */, {
        accessType: 'offline',
        authorizationURL: process.env.FACEBOOK_AUTH_URL,
        callbackURL: process.env.FACEBOOK_AUTH_CALLBACK_URL,
        clientID: process.env.FACEBOOK_AUTH_CLIENT_ID,
        clientSecret: process.env.FACEBOOK_AUTH_CLIENT_SECRET,
        tokenURL: process.env.FACEBOOK_AUTH_TOKEN_URL,
    }, queryGen('body')),
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)('/auth/facebook', {
        responses: {
            [200 /* OK */]: {
                description: 'POST Call for Facebook based login',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { 'x-ts-type': token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    (0, tslib_1.__param)(0, (0, rest_1.requestBody)({
        content: {
            'application/x-www-form-urlencoded': {
                schema: (0, rest_1.getModelSchemaRef)(client_auth_request_dto_1.ClientAuthRequest),
            },
        },
    })),
    (0, tslib_1.__metadata)("design:type", Function),
    (0, tslib_1.__metadata)("design:paramtypes", [client_auth_request_dto_1.ClientAuthRequest]),
    (0, tslib_1.__metadata)("design:returntype", Promise)
], FacebookLoginController.prototype, "postLoginViaFacebook", null);
(0, tslib_1.__decorate)([
    (0, loopback4_authentication_1.authenticate)("Facebook Oauth 2.0" /* FACEBOOK_OAUTH2 */, {
        accessType: 'offline',
        authorizationURL: process.env.FACEBOOK_AUTH_URL,
        callbackURL: process.env.FACEBOOK_AUTH_CALLBACK_URL,
        clientID: process.env.FACEBOOK_AUTH_CLIENT_ID,
        clientSecret: process.env.FACEBOOK_AUTH_CLIENT_SECRET,
        tokenURL: process.env.FACEBOOK_AUTH_TOKEN_URL,
    }, queryGen('query')),
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.get)('/auth/facebook-auth-redirect', {
        responses: {
            [200 /* OK */]: {
                description: 'Facebook Redirect Token Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { 'x-ts-type': token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    (0, tslib_1.__param)(0, rest_1.param.query.string('code')),
    (0, tslib_1.__param)(1, rest_1.param.query.string('state')),
    (0, tslib_1.__param)(2, (0, context_1.inject)(rest_1.RestBindings.Http.RESPONSE)),
    (0, tslib_1.__param)(3, (0, context_1.inject)(providers_1.AuthCodeBindings.CODEWRITER_PROVIDER)),
    (0, tslib_1.__param)(4, (0, context_1.inject)(loopback4_authentication_1.AuthenticationBindings.CURRENT_USER)),
    (0, tslib_1.__metadata)("design:type", Function),
    (0, tslib_1.__metadata)("design:paramtypes", [String, String, Object, Function, Object]),
    (0, tslib_1.__metadata)("design:returntype", Promise)
], FacebookLoginController.prototype, "facebookCallback", null);
FacebookLoginController = (0, tslib_1.__decorate)([
    (0, tslib_1.__param)(0, (0, repository_1.repository)(repositories_1.AuthClientRepository)),
    (0, tslib_1.__param)(1, (0, context_1.inject)(core_1.LOGGER.LOGGER_INJECT)),
    (0, tslib_1.__metadata)("design:paramtypes", [repositories_1.AuthClientRepository, Object])
], FacebookLoginController);
exports.FacebookLoginController = FacebookLoginController;
//# sourceMappingURL=facebook-login.controller.js.map